### AUTHORS ###################################################################
# Name			Date
# Gareth		2019-04-23
#

### DESCRIPTION ###############################################################
# script to process CREBBP MT/WT single cell RNASeq data and generatee cell trajectory plots

### LIBRARIES #################################################################
library(viridis)
library(monocle) # monocle_2.12.0

### FUNCTIONS #################################################################
calc.hypoxia.signature.buffa <- function(cds){
	
	# calculate signature score as mean of normalised expression
	hypoxia <- c("KIF20A", "CDKN3", "TUBA1B", "NDRG1", "MAD2L2", "YKT6", "ACOT7", 
		"LRRC42", "ADM", "CTSV", "ENO1", "AK4", "AK3L1", "ALDOA", "SEC61G", 
		"CORO1C", "KIF4A", "GAPDH", "GPI", "MCTS1", "MRPL13", "MRPL15", 
		"HILPDA", "HK2", "ANKRD37", "LDHA", "MIF", "PNP", "P4HA1", "UTP11L", 
		"CHCHD2", "MRPS17", "PFKP", "PGAM1", "PGK1", "ANLN", "DDIT4", 
		"ESRP1", "MRGBP", "MAP7D1", "PSMA7", "SLC2A1", "SLC16A1", "BNIP3", 
		"TPI1", "VEGFA", "CA9", "SHCBP1", "SLC25A32", "TUBB6", "PSRC1", 
		"TUBA1C")
	
	fData(cds)$hypoxia <- fData(cds)$gene_short_name %in% hypoxia
	nrm.dt <- t( t( exprs(cds) ) / sizeFactors(cds))
	nrm.dt <- log10(nrm.dt + 1)
	hypoxia.mat <- nrm.dt[rownames(fData(cds)[fData(cds)$hypoxia,]),]
	
	pData(cds)$Hypoxia <- rowMeans(t(hypoxia.mat[,rownames(pData(cds))]))
	
	return(cds)
}

read.scRNA <- function(count.dir, sample.name, count.file.suffix =  "/outs/matrix.filtered.csv"){
	
	tmp.count.dir <- paste0(count.dir, sample.name, count.file.suffix)
	cat("reading matrix from", tmp.count.dir, "\n")
	
	# read in matrix
	dat <- read.csv(tmp.count.dir, header = TRUE, row.names = 1)
	
	#read in cell data
	outs.dirs <- list.dirs(paste0(count.dir,sample.name,"/outs"), recursive = FALSE)
	base.dir <- paste0(outs.dirs[grep("*filtered_gene_bc_matrices", outs.dirs)], "/hg19/")
	cat("reading metadata from", base.dir,"\n")
	
	# read in gene annotations
	gene.ann <- paste0(base.dir, list.files(base.dir, pattern = "genes.tsv")[1])
	gene.ann <- read.table(gene.ann, row.names = 1)
	colnames(gene.ann) <- "gene_short_name"
	
	# read in cell barcodes
	pheno.ann <- paste0(base.dir, list.files(base.dir, pattern = "*barcodes.tsv")[1])
	pheno.ann <- read.table(pheno.ann)
	pheno.ann[,"cellName"]  <- make.names(pheno.ann[,"V1"])
	
	# make a new name for this cell and make these rownames
	pheno.ann[,"cellID"] <- paste0("C", sprintf("%04d", 1:nrow(pheno.ann)), "_", sample.name)
	rownames(pheno.ann) <- make.names(pheno.ann[,"cellID"])
	
	# add group column
	pheno.ann[,"Group"] <- sample.name 
	
	# drop the original cell id as names not checked
	pheno.ann <- pheno.ann[,-1]
	
	# change dat cell names to 
	colnames( dat ) <- pheno.ann[ match( colnames(dat), pheno.ann$cellName), "cellID" ]
	
	# make containers
	pd <- new("AnnotatedDataFrame", data = pheno.ann)
	fd <- new("AnnotatedDataFrame", data = gene.ann)
	
	# lists for return
	raw.dat <- list(
		pheno.ann = pheno.ann,
		gene.ann = gene.ann
	);
	
	monocle.dat <- list(
		dat = dat,
		pd = pd,
		fd = fd
	);
	
	return(
		list(raw = raw.dat, monocle = monocle.dat)
		);
}


build.monocle.obj <- function (sample.data) {
	cds <- newCellDataSet(
		cellData = as.matrix(sample.data[["monocle"]][["dat"]]), 
		phenoData = sample.data[["monocle"]][["pd"]], 
		featureData = sample.data[["monocle"]][["fd"]], 
		expressionFamily = negbinomial.size()
		);
	return(cds)
}

### MAIN ######################################################################
# Which sample to run analysis for, either "HAP1_CREBB_MUT" or "HAP1_WT"
pick.sample <-  "HAP1_CREBB_MUT" #"HAP1_WT" 

# cellranger outputs in this directory
count.dir <- "../../../../../../SHARED/data/FGENOMIC/Single_cell_RNA_seq/processed/GC-RN-7797/counts/"

sample.data <- read.scRNA(count.dir, pick.sample)

# conatiner for monocle
cds  <- build.monocle.obj(sample.data)

# Normalise and calc gene level dispersions
cds <- estimateSizeFactors(cds)
cds <- estimateDispersions(cds)

# Identify genes with at least 1 count in each cell
cds <- detectGenes(cds, min_expr = 1)

# Select genes for clustering
disp.table <- dispersionTable(cds)
clustering.genes <- subset(disp.table, mean_expression >= 0.1)
cds <- setOrderingFilter(cds, clustering.genes$gene_id) 

# Pseudotime 
cds <-	reduceDimension(
	cds,
	method = 'DDRTree',
	verbose = T
);

# order cells by pseudotime
cds <-	orderCells(cds)

# calculate hypoxia signature
cds <- calc.hypoxia.signature.buffa(cds) 

#########
# Plots #
#########
plot.dir <- 'Plots/'
dir.create(plot.dir,showWarnings = F)
pdf(paste0(plot.dir, pick.sample, "_cell_trajectory.pdf"), width = 5, height = 5)

plot_cell_trajectory(cds, color_by = "State",	cell_size = 0.85) + 
	theme(
		legend.title=element_text(size=10), 
		legend.text=element_text(size=6), 
		legend.key.height = grid::unit(0.25, "in")
		); 

plot_cell_trajectory(cds, color_by = "Pseudotime",	cell_size = 0.85)  +
	theme(
		legend.title=element_text(size=10), 
		legend.text=element_text(size=6), 
		legend.key.height = grid::unit(0.25, "in")) + 
	scale_color_viridis();

plot_cell_trajectory(cds, color_by = "Hypoxia",	cell_size = 0.85)  + 
	theme(
		legend.title=element_text(size=10), 
		legend.text=element_text(size=6), 
		legend.key.height = grid::unit(0.25, "in")) +
	scale_color_viridis(limits = c(0.32, 0.805));
dev.off()

