### AUTHORS ###################################################################
# Name			Date
# Gareth		2019-29-04
#

### DESCRIPTION ###############################################################
# script to determine if crebbp loss correlates with transcriptional divertisty 

### LIBRARIES #################################################################
library(DESeq2)
library(vegan)

### MAIN ######################################################################
data.directory <- "../../../../../../SHARED/analysis/RachaelNatrajan/CREBBP_transcriptional_diversity/Data/"

# prep samples
samples.crebbp.status.complete <- read.csv(paste0(data.directory,"BP09715_results.csv"), stringsAsFactors = FALSE)
samples.crebbp.status.complete$TCGA_ID.short <- make.names(samples.crebbp.status.complete$X)
samples.crebbp.status.complete$Patient <- substr(make.names(samples.crebbp.status.complete$X), 1,12)
samples.crebbp.status.complete$Status <- ifelse(is.na(samples.crebbp.status.complete$CREBBP.altered), "WT", "MT")

#############
# gene data # 
#############
cat('\nLoading Gene level data')
# obtain from firebrowse
genes.dat <- read.table(
  paste0(data.directory, "BRCA.rnaseqv2__illuminahiseq_rnaseqv2__unc_edu__Level_3__RSEM_genes_normalized__data.data.txt"), 
	stringsAsFactors = FALSE,
	header = TRUE, 
	sep = "\t",
	row.names = 1
  );

genes.dat <- genes.dat[-1,]
colnames(genes.dat) <- substr(colnames(genes.dat),1,15)
genes.sample.sheet <- data.frame(Patient = colnames(genes.dat), stringsAsFactors = FALSE)

genes.sample.sheet[genes.sample.sheet$Patient %in% samples.crebbp.status.complete$TCGA_ID.short, "Select.Sample" ] <- TRUE
genes.sample.sheet <- genes.sample.sheet[!is.na(genes.sample.sheet$Select.Sample),]

# add status
genes.sample.sheet <- cbind(genes.sample.sheet, samples.crebbp.status.complete[match(genes.sample.sheet$Patient, samples.crebbp.status.complete$TCGA_ID.short),c("CREBBP.altered", "Status")])

# subset full expression matrix of TNBC for Rachael in seperate request
genes.subset.dat <- genes.dat[,genes.sample.sheet$Patient]
genes.subset.dat <- apply(genes.subset.dat, 2, as.numeric); 
rownames(genes.subset.dat) <- rownames(genes.dat)
genes.subset.log <- log2(genes.subset.dat + 1)

#################
# isoforms data #
#################
cat('\nLoading Isoform level data')
# obtain from firebrowse
isoforms.dat <- read.table(
  paste0(data.directory, "BRCA.rnaseqv2__illuminahiseq_rnaseqv2__unc_edu__Level_3__RSEM_isoforms_normalized__data.data.txt"), 
  stringsAsFactors = FALSE,
  header = TRUE, 
  sep = "\t",
  row.names = 1
);

isoforms.dat <- isoforms.dat[-1,]
colnames(isoforms.dat) <- substr(colnames(isoforms.dat),1,15)
isoforms.sample.sheet <- data.frame(Patient = colnames(isoforms.dat), stringsAsFactors = FALSE)

isoforms.sample.sheet[isoforms.sample.sheet$Patient %in% samples.crebbp.status.complete$TCGA_ID.short, "Select.Sample" ] <- TRUE
isoforms.sample.sheet <- isoforms.sample.sheet[!is.na(isoforms.sample.sheet$Select.Sample),]

# add status
isoforms.sample.sheet <- cbind(isoforms.sample.sheet, samples.crebbp.status.complete[match(isoforms.sample.sheet$Patient, samples.crebbp.status.complete$TCGA_ID.short),c("CREBBP.altered", "Status")])

#############
# exon data #
#############
cat('\nLoading Exon level data')
# file is very large so load in the first couple of lines to pick the samples
# obtain from firebrowse
exon.counts.names <- read.table(
  paste0(data.directory, "BRCA.rnaseqv2__illuminahiseq_rnaseqv2__unc_edu__Level_3__exon_quantification__data.data.txt"),
	nrow = 2, 
	sep = "\t", 
	stringsAsFactors = FALSE,
	header = TRUE
  );

# merge column names which are in the crebbp status, and those which are raw_counts, this will give us all in the file
keep.tcga.ids =  substr(colnames(exon.counts.names),1,15) %in% samples.crebbp.status.complete$TCGA_ID.short
keep.raw.counts = as.logical(exon.counts.names[1,] == "raw_counts")

exon.counts <- read.table(
  paste0(data.directory, "BRCA.rnaseqv2__illuminahiseq_rnaseqv2__unc_edu__Level_3__exon_quantification__data.data.txt"),
	sep = "\t", 
	stringsAsFactors = FALSE,
	header = TRUE,
	colClasses = ifelse(keep.tcga.ids & keep.raw.counts, "character", "NULL")
  );

exon.sample.sheet <- data.frame(TCGA_ID = colnames(exon.counts), stringsAsFactors = FALSE)
exon.sample.sheet$TCGA_ID.short <- substr(exon.sample.sheet$TCGA_ID,1, 15)

# select samples which are in the CREBBP selection with juncton data
exon.sample.sheet[exon.sample.sheet$TCGA_ID.short %in% samples.crebbp.status.complete$TCGA_ID.short , "Select.Sample"] <- TRUE
exon.sample.sheet <- exon.sample.sheet[!is.na(exon.sample.sheet$Select.Sample),]

# add status
exon.sample.sheet <- cbind(exon.sample.sheet, samples.crebbp.status.complete[match(exon.sample.sheet$TCGA_ID.short, samples.crebbp.status.complete$TCGA_ID.short),c("CREBBP.altered", "Status")])
exon.counts <- exon.counts[-1,]

#################
# Gene Analysis # 
#################

# genes have all samples just like junctions so we will copy this
selected.genes.dat <- genes.dat[,genes.sample.sheet$Patient]
selected.genes.dat <- apply(selected.genes.dat, 2, as.numeric)

div.genes <- diversity(t(selected.genes.dat))
div.genes.eq <- div.genes / log(specnumber(t(selected.genes.dat)))

#####################
# isoforms Analysis # 
#####################

# genes have all samples just like junctions so we will copy this
selected.isoforms.dat <- isoforms.dat[,isoforms.sample.sheet$Patient]
selected.isoforms.dat <- apply(selected.isoforms.dat, 2, as.numeric)

div.isoforms <- diversity(t(selected.isoforms.dat))
div.isoforms.eq <- div.isoforms / log(specnumber(t(selected.isoforms.dat)))
	
#################
# Exon Analysis # 
#################

# extract the samples
selected.exons.dat <- exon.counts[,exon.sample.sheet$TCGA_ID]
selected.exons.dat <- apply(selected.exons.dat, 2, as.numeric)

# lets norm diversity
dds.exons <- DESeqDataSetFromMatrix(
	countData = selected.exons.dat, 
	colData = exon.sample.sheet, 
	design= ~1)
dds.exons <- estimateSizeFactors(dds.exons)
exons.counts.norm <- counts( dds.exons, normalized = TRUE) 

div.exons <- diversity(t(exons.counts.norm))
div.exons.eq <- div.exons / log(specnumber(t(exons.counts.norm)))

#########
# Stats # 
#########
cat("\nGene level")
t.test(div.genes.eq ~ genes.sample.sheet$Status)
cat("\nIsoform level")
t.test(div.isoforms.eq ~ isoforms.sample.sheet$Status)
cat("\nExon level")
t.test(div.exons.eq ~ exon.sample.sheet$Status)

#########
# Plots # 
#########
plot.dir <- 'Plots/'
dir.create(plot.dir,showWarnings = F)

pdf(paste0(plot.dir, "Transcriptional_heterogeneity.pdf"), width = 5, height = 5)
boxplot(div.genes.eq ~ genes.sample.sheet$Status, main = "Gene")
boxplot(div.isoforms.eq ~ isoforms.sample.sheet$Status, main = "Isoform")
boxplot(div.exons.eq ~ exon.sample.sheet$Status, main = "Exons")
dev.off()




